// Copyright (c) 2022 ConvivaAppAnalytics All rights reserved.
//
// This program is licensed to you under the Apache License Version 2.0,
// and you may not use this file except in compliance with the Apache License Version 2.0.
// You may obtain a copy of the Apache License Version 2.0 at http://www.apache.org/licenses/LICENSE-2.0.
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the Apache License Version 2.0 is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the Apache License Version 2.0 for the specific language governing permissions and limitations there under.

package com.conviva.flutter.apptracker

import android.content.Context

import com.conviva.apptracker.ConvivaAppAnalytics
import com.conviva.apptracker.configuration.Configuration
import com.conviva.flutter.apptracker.readers.configurations.DefaultTrackerConfiguration
import com.conviva.flutter.apptracker.readers.messages.*

object ConvivaTrackerController {

    fun createTracker(messageReader: CreateTrackerMessageReader, context: Context) {
        val controllers: MutableList<Configuration> = mutableListOf()

        val appName = messageReader.appName
        val customerKey = messageReader.customerKey

        val networkConfigReader = messageReader.networkConfig
        networkConfigReader?.let { controllers.add(it.toConfiguration()) }

        val trackerConfigReader = messageReader.trackerConfig
        if (trackerConfigReader == null) {
            controllers.add(DefaultTrackerConfiguration.toConfiguration(appName, context))
        } else {
            controllers.add(trackerConfigReader.toConfiguration(appName, context))
        }

        val subjectConfigReader = messageReader.subjectConfig
        subjectConfigReader?.let { controllers.add(it.toConfiguration()) }

        val gdprConfigReader = messageReader.gdprConfig
        gdprConfigReader?.let { controllers.add(it.toConfiguration()) }

        val emitterConfigReader = messageReader.emitterConfig
        emitterConfigReader?.let { controllers.add(it.toConfiguration()) }

        ConvivaAppAnalytics.createTracker(
                context,
                customerKey,
                appName,
                *controllers.toTypedArray()
        )
    }

    fun trackCustomEvent(messageReader: TrackCustomEventMessageReader) {
        val trackerController = ConvivaAppAnalytics.getTracker(messageReader.tracker)

        trackerController?.trackCustomEvent(messageReader.name, messageReader.data ?: "")
    }

    fun trackStructured(eventReader: EventMessageReader) {
        val trackerController = ConvivaAppAnalytics.getTracker(eventReader.tracker)
        val structured = eventReader.toStructuredWithContexts()

        trackerController?.track(structured)
    }

    fun trackSelfDescribing(eventReader: EventMessageReader) {
        val trackerController = ConvivaAppAnalytics.getTracker(eventReader.tracker)
        val selfDescribing = eventReader.toSelfDescribingWithContexts()

        trackerController?.track(selfDescribing)
    }

    fun trackScreenView(eventReader: EventMessageReader) {
        val trackerController = ConvivaAppAnalytics.getTracker(eventReader.tracker)
        val screenView = eventReader.toScreenViewWithContexts()

        trackerController?.track(screenView)
    }

    fun trackTiming(eventReader: EventMessageReader) {
        val trackerController = ConvivaAppAnalytics.getTracker(eventReader.tracker)
        val timing = eventReader.toTimingWithContexts()

        trackerController?.track(timing)
    }

    fun trackConsentGranted(eventReader: EventMessageReader) {
        val trackerController = ConvivaAppAnalytics.getTracker(eventReader.tracker)
        val consentGranted = eventReader.toConsentGrantedWithContexts()

        trackerController?.track(consentGranted)
    }

    fun trackConsentWithdrawn(eventReader: EventMessageReader) {
        val trackerController = ConvivaAppAnalytics.getTracker(eventReader.tracker)
        val consentWithdrawn = eventReader.toConsentWithdrawnWithContexts()

        trackerController?.track(consentWithdrawn)
    }

    fun setUserId(messageReader: SetUserIdMessageReader) {
        val trackerController = ConvivaAppAnalytics.getTracker(messageReader.tracker)

        trackerController?.subject?.userId = messageReader.userId
    }

    fun getSessionUserId(messageReader: GetParameterMessageReader): String? {
        val trackerController = ConvivaAppAnalytics.getTracker(messageReader.tracker)

        return trackerController?.session?.userId
    }

    fun getSessionId(messageReader: GetParameterMessageReader): String? {
        val trackerController = ConvivaAppAnalytics.getTracker(messageReader.tracker)

        return trackerController?.session?.sessionId
    }

    fun getSessionIndex(messageReader: GetParameterMessageReader): Int? {
        val trackerController = ConvivaAppAnalytics.getTracker(messageReader.tracker)

        return trackerController?.session?.sessionIndex
    }

}
