// Copyright (c) 2022 Conviva All rights reserved.
//
// This program is licensed to you under the Apache License Version 2.0,
// and you may not use this file except in compliance with the Apache License Version 2.0.
// You may obtain a copy of the Apache License Version 2.0 at http://www.apache.org/licenses/LICENSE-2.0.
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the Apache License Version 2.0 is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the Apache License Version 2.0 for the specific language governing permissions and limitations there under.

import Foundation
import ConvivaAppAnalytics

class ConvivaAppAnalyticsController {
    static func createTracker(_ message: CreateTrackerMessageReader, arguments: [String: Any]) {
        var controllers: [CATConfiguration] = []

        if let trackerConfig = message.trackerConfig {
            controllers.append(trackerConfig.toConfiguration(appName: message.appName))
        } else {
            controllers.append(TrackerConfigurationReader.defaultConfiguration(appName: message.appName))
        }
        if let subjectConfig = message.subjectConfig {
            controllers.append(subjectConfig.toConfiguration())
        }
        if let gdprConfig = message.gdprConfig {
            controllers.append(gdprConfig.toConfiguration())
        }
        if let emitterConfig = message.emitterConfig {
            controllers.append(emitterConfig.toConfiguration())
        }
        
        if let networkConfig = message.networkConfig, let networkConfiguration = networkConfig.toConfiguration() {
            controllers.append(networkConfiguration)
        }

        CATAppAnalytics.createTracker(customerKey: message.customerKey, appName: message.appName, configurations: controllers)

    }

    static func trackStructured(_ message: TrackStructuredMessageReader, eventMessage: EventMessageReader, arguments: [String: Any]) {
        let event = message.toStructured()
        trackEvent(event, eventMessage: eventMessage, arguments: arguments)
    }

    static func trackSelfDescribing(_ message: TrackSelfDescribingMesssageReader, eventMessage: EventMessageReader, arguments: [String: Any]) {
        if let event = message.toSelfDescribing(arguments: arguments) {
            trackEvent(event, eventMessage: eventMessage, arguments: arguments)
        }
    }

    static func trackScreenView(_ message: TrackScreenViewMessageReader, eventMessage: EventMessageReader, arguments: [String: Any]) {
        let event = message.toScreenView()
        trackEvent(event, eventMessage: eventMessage, arguments: arguments)
    }
    
    static func trackPageView(_ message: TrackPageViewMessageReader, eventMessage: EventMessageReader, arguments: [String: Any]) {
        let event = message.toPageView()
        trackEvent(event, eventMessage: eventMessage, arguments: arguments)
    }

    static func trackTiming(_ message: TrackTimingMessageReader, eventMessage: EventMessageReader, arguments: [String: Any]) {
        let event = message.toTiming()
        trackEvent(event, eventMessage: eventMessage, arguments: arguments)
    }

    static func trackConsentGranted(_ message: TrackConsentGrantedMessageReader, eventMessage: EventMessageReader, arguments: [String: Any]) {
        let event = message.toConsentGranted()
        trackEvent(event, eventMessage: eventMessage, arguments: arguments)
    }

    static func trackConsentWithdrawn(_ message: TrackConsentWithdrawnMessageReader, eventMessage: EventMessageReader, arguments: [String: Any]) {
        let event = message.toConsentWithdrawn()
        trackEvent(event, eventMessage: eventMessage, arguments: arguments)
    }

    static func sessionUserId(_ message: GetParameterMessageReader) -> String? {
        return CATAppAnalytics.tracker(namespace: message.tracker)?.session?.userId
    }

    static func sessionId(_ message: GetParameterMessageReader) -> String? {
        return CATAppAnalytics.tracker(namespace: message.tracker)?.session?.sessionId
    }

    static func sessionIndex(_ message: GetParameterMessageReader) -> Int? {
        return CATAppAnalytics.tracker(namespace: message.tracker)?.session?.sessionIndex
    }

    static func setUserId(_ message: SetUserIdMessageReader) {
        let trackerController = CATAppAnalytics.tracker(namespace: message.tracker)
        trackerController?.subject?.userId = message.userId
    }

    static func trackCustomEvent(_ message: TrackCustomEventMessageReader) {
        let trackerController = CATAppAnalytics.tracker(namespace: message.tracker)
        trackerController?.trackCustomEvent(message.name, data: message.data!)
    }

    static func setCustomTags(_ message: CustomTagsMessageReader) {

        let trackerController = CATAppAnalytics.tracker(namespace: message.tracker)

        if let _ = message.tags, let dataTags = message.convertToDictionary(text: message.tags!) {
            trackerController?.setCustomTags(dataTags)
        }
    }

    static func clearCustomTags(_ message: CustomTagsMessageReader) {
        let trackerController = CATAppAnalytics.tracker(namespace: message.tracker)

        if let _ = message.clearTags {
            trackerController?.clearCustomTags(message.clearTags!)
        }
    }

    static func clearAllCustomTags(_ message: CustomTagsMessageReader) {

        let trackerController = CATAppAnalytics.tracker(namespace: message.tracker)

        trackerController?.clearCustomTags()
        
        
    }

    private static func trackEvent(_ event: CATEvent, eventMessage: EventMessageReader, arguments: [String: Any]) {
        eventMessage.addContextsToEvent(event, arguments: arguments)
        let trackerController = CATAppAnalytics.tracker(namespace: eventMessage.tracker)
        trackerController?.track(event)
    }
}
